(
SynthDef(\hierarchical_dust, {
	arg stable = 10, unstable = 10, buf = 0, loop = 0, cycle_len = 10;
	var local_in, hold, change, state, latch, hierarchical_dust, generator, env_master, env_spectrum, env_vol, chain, spectrum_mult;

	// Feedback in state
	local_in = LocalIn.kr(2, 0);
	// Make each state last at least 15 seconds, however these variables could be different / manipulated in the array [15, 15, 15]
	// That is, they could be turned into user variables
	// Note the second two account for the fade in and fade out of the necklace, so it will be at least 30 seconds long
	hold = PulseCount.kr(Impulse.kr(1), Changed.kr(local_in[0])) > Select.kr(local_in[0], [15, 15, 15]);
	// Change state trigger
	change = TWChoose.kr(Impulse.kr(1), [0, 1], [Select.kr(local_in[0] > 0, [unstable, stable]), 1].normalizeSum) * hold;
	// Change state
	state = Stepper.kr(change + TDelay.kr(local_in[0] > 1, 15), 0, 0, 2);

	// Monitor
	Poll.kr(Impulse.kr(1), hold, \hold);
	Poll.kr(Impulse.kr(1), change, \change);
	Poll.kr(Impulse.kr(1), state, \state);

	// Trigger for filter changes
	hierarchical_dust = (
		Impulse.kr(8) *
		(TRand.kr(0, 1, Impulse.kr(8)) <= 0.5)  *
		(TRand.kr(0, 1, Impulse.kr(1)) <= 0.75) *
		(state <= 0)
	);

	// Playback the soundfile
	generator = PlayBuf.ar(1, buf, BufRateScale.kr(buf), 1, 0, 1);
	latch = Impulse.kr(Latch.kr((60 * cycle_len).reciprocal, local_in[1]));
	env_master = EnvGen.kr(Env.sine(Latch.kr(60 * cycle_len, latch)), latch * (loop + Impulse.kr(0)));
	Poll.kr(Impulse.kr(1), env_master, \env);

	// Feedback out state
	LocalOut.kr([state, latch]);

	// Gate bins of the FFT
	env_spectrum = pow(env_master, 3) * 0.75;
	chain= FFT(LocalBuf(128).clear, generator);
	spectrum_mult = { |i| TRand.kr(0, 1, hierarchical_dust) > Latch.kr(env_spectrum, hierarchical_dust)} ! 64;
	chain = chain.pvcalc(64, {|mags, phases| [mags * spectrum_mult, phases] });

	// Output
	env_vol = pow(env_master, 2);
	Out.ar([0,1], IFFT(chain).dup * env_vol * (1 - EnvGen.kr(Env.asr(15, 0.9, 15, \sine), state % 2)));

	// Send info to Visuals
	SendTrig.kr(Impulse.kr(24), 0,
		1 - EnvGen.kr(Env.sine(1/6.0, env_vol), Select.kr(state > 0, [hierarchical_dust, state - Delay1.kr(state) < 0])));
	SendTrig.kr((state - Delay1.kr(state) < 0) * PulseCount.kr(Changed.kr(local_in[0])) >= 1, 1);
	SendTrig.kr(Changed.kr(state), 2, state > 0);
	SendReply.kr(Impulse.kr(24), '/tr', spectrum_mult, 3);
}).send(s);
)